import csv
import datetime
from dataclasses import dataclass
from pathlib import Path
from babel.dates import format_date
from jinja2 import Environment, PackageLoader, select_autoescape
from typing import Union
import subprocess


INSTITUTION_SHORTCUTS = {
    "ICL": "ICL, Heidelberg University",
    "HITS": "Heidelberg Institute for Theoretical Studies (HITS)"
}


@dataclass
class Talk:
    date: datetime.date
    speaker: str
    speaker_institution: str
    speaker_link: str
    title: str
    abstract: str

    @property
    def formatted_date(self) -> str:
        return format_date(self.date, "dd. MMMM yyyy", locale='en')

    @property
    def abstract_filename(self) -> str:
        date_part = format_date(self.date, "dd_MM_yyyy", locale='en')
        return f"{date_part}.mhtml"

    @property
    def abstract_url(self) -> str:
        return f"docs/{self.abstract_filename}"


def get_if_exists(d: dict[str, str], key: str) -> str:
    s = d.get(key)
    if s is None:
        return None
    s = s.strip()
    if len(s) == 0:
        return None
    else:
        return s


def read_schedule(path: Union[Path, str]) -> list[Talk]:
    path = Path(path)
    with open(path) as f:
        s = csv.Sniffer()
        csv_dialect = s.sniff(path.read_text())
        parser = csv.DictReader(f, fieldnames=[
            "Date",
            "Speaker",
            "Institution",
            "Link",
            "Title",
            "Abstract"
            ],
            dialect=csv_dialect
        )
        next(parser) # Skip header

        talks = []
        for entry in parser:
            date = get_if_exists(entry, "Date")
            if date is None:
                continue

            speaker = get_if_exists(entry, "Speaker")
            if speaker is None:
                continue

            date = datetime.datetime.strptime(date, r"%d/%m/%Y").date()
            speaker_institution = get_if_exists(entry, "Institution")
            speaker_institution = INSTITUTION_SHORTCUTS.get(speaker_institution, speaker_institution)

            talks.append(
                Talk(
                    date=date,
                    speaker=speaker,
                    speaker_institution=speaker_institution,
                    speaker_link=get_if_exists(entry, "Link"),
                    title=get_if_exists(entry, "Title"),
                    abstract=get_if_exists(entry, "Abstract"),
                )
            )
    
    return talks


def main():
    talks = read_schedule("schedule.csv")
    env = Environment(
        loader=PackageLoader("colloqtool"),
        autoescape=select_autoescape()
    )

    schedule_template = env.get_template("schedule.html")
    abstract_template = env.get_template("abstract.html")

    main_table = schedule_template.render(talks=talks)
    Path("index_table_cl_colloquium.shtml").write_text(main_table)
    abstracts_path = Path("docs")
    abstracts_path.mkdir(exist_ok=True)

    for talk in talks:
        abstract_path = (abstracts_path / talk.abstract_filename)
        if abstract_path.exists():
            if talk.abstract is None:
                abstract_path.unlink()
            continue

        if talk.abstract is None:
            continue

        abstract_site = abstract_template.render(talk=talk)
        abstract_path.write_text(abstract_site)

    # /var/www/htdocs/colloquium/cl_colloquium/docs/
    import shlex
    print("Running rsync…")
    subprocess.run(['rsync', '-rv', 'docs', "index_table_cl_colloquium.shtml", 'reed:/var/www/htdocs/colloquium/cl_colloquium/'], capture_output=False, shell=False, text=True, stderr=subprocess.STDOUT)

if __name__ == "__main__":
    main()
